__author__ = 'Kelly Boyd and Emma Highland'

import sys
import os


# Set up
def setup():
    params_list = []  # initiate an empty list which will contain all parameters
    my_file = "RangerParameters.txt"  # file which contains all parameters
    fd = open(my_file, 'r')
    for line in fd:  # reading each line to check for appropriate parameter title
        if line.startswith("[RANGEROUT]"):
            if line != "":
                params_list.append(line.split(",")[1])
            else:
                print("ERROR.")
                sys.exit()
        if line.startswith("[DBPATH]"):
            if line != "":
                params_list.append(line.split(",")[1])
            else:
                print("ERROR.")
                sys.exit()
        if line.startswith("[EVALUE]"):
            if line != "":
                params_list.append(line.split(",")[1])
            else:
                print("ERROR.")
                sys.exit()
        if line.startswith("[MAXTARGSEQ]"):
            if line != "":
                params_list.append(line.split(",")[1])
            else:
                print("ERROR.")
                sys.exit()
        if line.startswith("[QUERY]"):
            if line != "":
                params_list.append(line.split(",")[1])
            else:
                print("ERROR.")
                sys.exit()
        if line.startswith("[OUTFMT]"):
            if line != "":
                params_list.append(line.split(",")[1])
            else:
                print("ERROR.")
                sys.exit()
        if line.startswith("[NUMTHREAD]"):
            if line != "":
                params_list.append(line.split(",")[1])
            else:
                print("ERROR.")
                sys.exit()
        if line.startswith("[OPTPATH]"):
            if line != "":
                params_list.append(line.split(",")[1])
            else:
                print("ERROR.")
        if line.startswith("[OPTDIR]"):
            if line != "":
                params_list.append(line.split(",")[1])
            else:
                print("ERROR.")
    fd.close()

    return params_list  # return the full list so it can be used later


# Run VelvetOptimiser
def run_optimiser(my_file):  # grabs the appropriate file for input
    my_list = setup()  # grabs the set up list
    optimiser = my_list[7]  # grabs the appropriate parameters
    output = my_list[8]

    my_run = "perl " + optimiser + " -f" + " -fastq " + str(my_file) + " > " + output
    # creates the variable "my_run" which will be executed in the command line
    # " > " + output ensures that the name of the logfile generated by VelvetOptimiser will be saved in a text file
    os.system(my_run)
    # my_run is executed in the command line

    # initializes two lists, one of which will hold the name of the VelvetOptimiser log file
    # and of which will hold the VelvetOptimiser ID which corresponds to the best k-mer value
    my_log = []
    my_id = []

    fd = open(output, "r")  # opens the txt file that holds the name of the log file
    for line in fd:
        if "Logfile name: " in line:  # checks to make sure the output text file contains the name of the log file
            my_log.append(line[14:45])
            # appends the name of the logfile (which is contained between position 14 and 45)
            my_file = str(my_log)
            fd2 = open(my_file[2:33], "r")  # opens the original logfile generated by VelvetOptimiser
            for line2 in fd2:
                if "Best assembly by assembly score - assembly id: " in line2:
                    # looks for the above line
                    my_id.append(line2[63])  # appends the ID number (contained at position 63) to the my_id array
                    if my_id[0] == "1":  # each ID number (1-7) corresponds to the values 19-31
                        return "19"  # returns the appropriate ID number
                    if my_id[0] == "2":
                        return "21"
                    if my_id[0] == "3":
                        return "23"
                    if my_id[0] == "4":
                        return "25"
                    if my_id[0] == "5":
                        return "27"
                    if my_id[0] == "6":
                        return "29"
                    if my_id[0] == "7":
                        return "31"
                    else:
                        print("ERROR.")
            fd2.close()
    fd.close()
    # close each read file


# Run Velveth
def run_velveth(my_file, folder):  # grabs the appropriate file for input and folder for output
    my_list = setup()  # grabs the list from set up
    velvetout = my_list[0]  # grabs the appropriate parameter
    kmer_length = run_optimiser(my_file)  # gets the k-mer length from the output of run_optimiser

    my_run = "velveth " + velvetout + "/" + folder + " " + kmer_length + " -fastq " + str(my_file)
    # creates the variable "my_run" which will be executed in the command line
    os.system(my_run)
    # my_run is executed in the command line


# Run Velvetg
def run_velvetg(folder):  # grabs the appropriate folder for output
    my_list = setup()  # grabs the list from set up
    velvetout = my_list[0]  # grabs the appropriate parameter

    my_run = "sudo velvetg " + velvetout + "/" + folder
    # creates the variable "my_run" which will be executed in the command line
    # velvetg will not run unless "sudo" is used
    # user should be prepared to enter their password
    os.system(my_run)
    # my_run is executed in the command line


# Run BLASTx
def run_blastx(folder):  # grabs the appropriate folder for output
    my_list = setup()  # grabs the list from set up
    output = my_list[0]  # grabs the appropriate parameter
    database = my_list[1]
    evalue = my_list[2]
    targetseq = my_list[3]
    query = my_list[4]
    outfmt = my_list[5]
    cores = my_list[6]

    my_run = "sudo blastx -db " + database + " -evalue " + evalue + " -max_target_seqs " \
             + targetseq + " -query " + output + "/" + folder + "/" + \
             query + " -outfmt " + outfmt + " -out " + output + "/" + folder \
             + "/blast.out " + " -num_threads " + cores
    # creates the variable "my_run" which will be executed in the command line
    os.system(my_run)
    # my_run is executed in the command line


# Run the BLAST parser
def run_blastparse(folder):  # grabs the appropriate folder for output
    my_list = setup()  # grabs the list from set up
    output = my_list[0]  # grabs the appropriate parameter
    in_filepath = output + "/" + folder + "/blast.out"
    out_filepath = output + "/" + folder + "/blasthits.out"
    # create two filepaths, one for input and one for output

    my_run = str(in_filepath) + " " + str(out_filepath)
    # creates the variable "my_run" which will be executed in the command line
    os.system("./bparse " + my_run)
    # my_run is executed in the command line


def main():
    files = []  # empty list that will contain all the file paths
    folders = []  # empty list that will contain all the folder names
    my_list = setup()  # grabs the list from set up
    results = my_list[0]  # grabs the appropriate parameter

    yes_commands = ["Y", "y", "yes", "ye", "Yes", "Ye", "YES", "YE"]  # list of inputs that will be processed as "yes"
    no_commands = ["N", "n", "no", "No", "nO", "NO"]  # list of inputs that will be processed as "no"

    fd = open('RangerFolders.txt', 'r')  # open the "RangerFolders.txt file for reading
    for line in fd:
        if line.startswith("[RANGERLIST]"):  # look for the list of files to run through PyRANGER
            if line != "":
                files.append(line.split(",")[1:])
            else:
                print("ERROR.")
                sys.exit()
        if line.startswith("[FOLDERS]"):  # look for the desired folder names
            if line != "":
                folders.append(line.split(",")[1:])
            else:
                print("ERROR.")
                sys.exit()

    max_val = files[0].__len__() + 1  # maximum value is used later to specify how many times PyRANGER iterates
    # the maximum value is the length of the files list plus one to ensure that all files run

    print("**************************\n"
          "~~~Welcome to PyRANGER!~~~\n"
          "**************************\n")
    print("Have you filled out the config files?\n"
          "(RangerParameters.txt and RangerFolders.txt)")
    my_input = raw_input("[Y/N]\n")  # prompts for a yes/no answer about whether all config files have been completed
    if my_input in no_commands:
        print("Please fill out the config files and try again.")
        sys.exit()
        # PyRANGER will quit if the user states that the config files have not been completed
    elif my_input in yes_commands:
        for i in range(max_val):  # iterates through the list of files
            my_file = files[0][i]
            my_folder = folders[0][i]
            # PyRANGER grabs the appropriate files from the list. All files are in their own sub-lists.
            # PyRANGER begins to iterate:
            print("*********************\n"
                  "Beginning Velveth\n"
                  "and VelvetOptimiser\n"
                  "*********************")
            run_velveth(my_file, my_folder)  # run_velveth calls run_optimiser, so only run_velveth is called in main
            print("*********************\n"
                  "Velveth complete!\n"
                  "Beginning Velvetg\n"
                  "*********************")
            run_velvetg(my_folder)  # run_velvetg is called
            print("*********************\n"
                  "Velvetg complete!\n"
                  "Beginning blastx\n"
                  "*********************")
            run_blastx(my_folder)  # run_blastx is called
            print("*********************\n"
                  "blastx complete!\n"
                  "Beginning blastparse\n"
                  "*********************")
            run_blastparse(my_folder)  # run_blastparse is called
            print("*********************\n"
                  "Run " + str(i + 1) + " complete!\n"  # PyRANGER will tell the user when the runs complete
                  "Results are stored in " + results + "/" + my_folder + "\n"  # Location of the run's output files
                  "*********************")
    else:
        print("Invalid input. Try again.")  # if the input is not in yes_commands or no_commands, PyRANGER will quit
        sys.exit()
    fd.close()  # close the read file

main()  # run PyRANGER
